@@ -62,8 +62,8 @@ def test_blockcache_workflow(ftp_writable, tmp_path):
     # Check that cache file only has the first two blocks
     with open(tmp_path / "cache", "rb") as f:
         cache = pickle.load(f)
-        assert "ftp:///out" in cache
-        assert cache["ftp:///out"]["blocks"] == [0, 1]
+        assert "/out" in cache
+        assert cache["/out"]["blocks"] == [0, 1]
 
     # Reopen the same cache and read some more...
     fs = fsspec.filesystem("blockcache", **fs_kwargs)
@@ -88,9 +88,9 @@ def test_workflow(ftp_writable, impl):
     with fs.open("/out") as f:
         assert os.listdir(fs.storage[-1])
         assert f.read() == b"test"
-        assert fs.cached_files[-1]["ftp:///out"]["blocks"]
+        assert fs.cached_files[-1]["/out"]["blocks"]
     assert fs.cat("/out") == b"test"
-    assert fs.cached_files[-1]["ftp:///out"]["blocks"] is True
+    assert fs.cached_files[-1]["/out"]["blocks"] is True
 
     with fs.open("/out", "wb") as f:
         f.write(b"changed")
@@ -214,6 +214,77 @@ def test_blocksize(ftp_writable):
         fs.open("/out_block", block_size=30)
 
 
+def test_blockcache_multiinstance(ftp_writable):
+    host, port, user, pw = ftp_writable
+    fs = FTPFileSystem(host, port, user, pw)
+    with fs.open("/one", "wb") as f:
+        f.write(b"test" * 40)
+    with fs.open("/two", "wb") as f:
+        f.write(b"test" * 40)
+    fs = fsspec.filesystem(
+        "blockcache",
+        target_protocol="ftp",
+        target_options={"host": host, "port": port, "username": user, "password": pw},
+    )
+
+    with fs.open("/one", block_size=20) as f:
+        assert f.read(1) == b"t"
+    fs2 = fsspec.filesystem(
+        "blockcache",
+        target_protocol="ftp",
+        target_options={"host": host, "port": port, "username": user, "password": pw},
+        skip_instance_cache=True,
+        cache_storage=fs.storage,
+    )
+    assert fs2.cached_files  # loaded from metadata for "one"
+    with fs2.open("/two", block_size=20) as f:
+        assert f.read(1) == b"t"
+    assert "/two" in fs2.cached_files[-1]
+    fs.save_cache()
+    assert list(fs.cached_files[-1]) == ["/one", "/two"]
+    assert list(fs2.cached_files[-1]) == ["/one", "/two"]
+
+
+def test_metadata_save_blocked(ftp_writable, caplog):
+    import logging
+
+    host, port, user, pw = ftp_writable
+    fs = FTPFileSystem(host, port, user, pw)
+    with fs.open("/one", "wb") as f:
+        f.write(b"test" * 40)
+    fs = fsspec.filesystem(
+        "blockcache",
+        target_protocol="ftp",
+        target_options={"host": host, "port": port, "username": user, "password": pw},
+    )
+
+    with fs.open("/one", block_size=20) as f:
+        assert f.read(1) == b"t"
+    fn = os.path.join(fs.storage[-1], "cache")
+    with caplog.at_level(logging.DEBUG):
+        with fs.open("/one", block_size=20) as f:
+            f.seek(21)
+            assert f.read(1)
+            os.remove(fn)
+            os.mkdir(fn)
+    assert "Cache saving failed while closing file" in caplog.text
+    os.rmdir(fn)
+
+    def open_raise(*_, **__):
+        raise NameError
+
+    try:
+
+        with caplog.at_level(logging.DEBUG):
+            with fs.open("/one", block_size=20) as f:
+                fsspec.implementations.cached.open = open_raise
+                f.seek(21)
+                assert f.read(1)
+    finally:
+        fsspec.implementations.cached.__dict__.pop("open", None)
+    assert "Cache save failed due to interpreter shutdown" in caplog.text
+
+
 @pytest.mark.parametrize("impl", ["filecache", "simplecache", "blockcache"])
 def test_local_filecache_creates_dir_if_needed(impl):
     import tempfile
@@ -223,24 +294,18 @@ def test_local_filecache_creates_dir_if_needed(impl):
     os.rmdir(cache_location)
     assert not os.path.exists(cache_location)
 
-    try:
-        original_file = os.path.join(original_location, "afile")
-        data = b"test data"
-        with open(original_file, "wb") as f:
-            f.write(data)
+    original_file = os.path.join(original_location, "afile")
+    data = b"test data"
+    with open(original_file, "wb") as f:
+        f.write(data)
 
-        # we can access the file and read it
-        fs = fsspec.filesystem(
-            impl, target_protocol="file", cache_storage=cache_location
-        )
+    # we can access the file and read it
+    fs = fsspec.filesystem(impl, target_protocol="file", cache_storage=cache_location)
 
-        with fs.open(original_file, "rb") as f:
-            data_in_cache = f.read()
+    with fs.open(original_file, "rb") as f:
+        data_in_cache = f.read()
 
-        assert os.path.exists(cache_location)
-
-    finally:
-        shutil.rmtree(cache_location)
+    assert os.path.exists(cache_location)
 
     assert data_in_cache == data
 
@@ -558,6 +623,19 @@ def test_multi_cache(protocol):
             assert f.read() == b"hello"
 
 
+@pytest.mark.parametrize("protocol", ["simplecache", "filecache", "blockcache"])
+def test_multi_cat(protocol, ftp_writable):
+    host, port, user, pw = ftp_writable
+    fs = FTPFileSystem(host, port, user, pw)
+    for fn in {"/file0", "/file1"}:
+        with fs.open(fn, "wb") as f:
+            f.write(b"hello")
+
+    d2 = tempfile.mkdtemp()
+    fs = fsspec.filesystem(protocol, storage=d2, fs=fs)
+    assert fs.cat("file*") == {"/file0": b"hello", "/file1": b"hello"}
+
+
 @pytest.mark.parametrize("protocol", ["simplecache", "filecache"])
 def test_multi_cache_chain(protocol):
     import zipfile
@@ -595,3 +673,47 @@ def test_cached_write(protocol):
             f.write(b"data")
 
     assert sorted(os.listdir(d)) == ["0.out", "1.out"]
+
+
+def test_expiry():
+    import time
+
+    d = tempfile.mkdtemp()
+    fs = fsspec.filesystem("memory")
+    fn = "afile"
+    fn0 = "memory://afile"
+    data = b"hello"
+    with fs.open(fn0, "wb") as f:
+        f.write(data)
+
+    fs = fsspec.filesystem(
+        "filecache",
+        fs=fs,
+        cache_storage=d,
+        check_files=False,
+        expiry_time=0.1,
+        same_names=True,
+    )
+
+    # get file
+    assert fs._check_file(fn0) is False
+    assert fs.open(fn0, mode="rb").read() == data
+    start_time = fs.cached_files[-1][fn]["time"]
+
+    # cache time..
+    assert fs.last_cache - start_time < 0.19
+
+    # cache should have refreshed
+    time.sleep(0.01)
+
+    # file should still be valid... re-read
+    assert fs.open(fn0, mode="rb").read() == data
+    detail, _ = fs._check_file(fn0)
+    assert detail["time"] == start_time
+
+    time.sleep(0.11)
+    # file should still be invalid... re-read
+    assert fs._check_file(fn0) is False
+    assert fs.open(fn0, mode="rb").read() == data
+    detail, _ = fs._check_file(fn0)
+    assert detail["time"] - start_time > 0.09
