@@ -125,6 +125,28 @@ def test_read(server):
         assert f.read(100) + f.read() == data
 
 
+def test_file_pickle(server):
+    import pickle
+
+    # via HTTPFile
+    h = fsspec.filesystem("http", headers={"give_length": "true", "head_ok": "true"})
+    out = server + "/index/realfile"
+    with h.open(out, "rb") as f:
+        pic = pickle.dumps(f)
+        assert f.read() == data
+    with pickle.loads(pic) as f:
+        assert f.read() == data
+
+    # via HTTPStreamFile
+    h = fsspec.filesystem("http")
+    out = server + "/index/realfile"
+    with h.open(out, "rb") as f:
+        out = pickle.dumps(f)
+        assert f.read() == data
+    with pickle.loads(out) as f:
+        assert f.read() == data
+
+
 def test_methods(server):
     h = fsspec.filesystem("http")
     url = server + "/index/realfile"
@@ -205,6 +227,13 @@ def test_mcat(server):
     assert out == {urla: data, urlb: data}
 
 
+def test_mcat_cache(server):
+    urla = server + "/index/realfile"
+    urlb = server + "/index/otherfile"
+    fs = fsspec.filesystem("simplecache", target_protocol="http")
+    assert fs.cat([urla, urlb]) == {urla: data, urlb: data}
+
+
 def test_mcat_expand(server):
     h = fsspec.filesystem("http", headers={"give_length": "true", "head_ok": "true "})
     out = h.cat(server + "/index/*")
@@ -265,7 +294,7 @@ def _inner_pass(fs, q, fn):
 
 
 @pytest.mark.skipif(
-    bool(os.environ.get("TRAVIS", "")), reason="Travis is weird in many ways"
+    bool(os.environ.get("CIRUN", "")), reason="CI runs are weird in many ways"
 )
 @pytest.mark.parametrize("method", ["spawn", "forkserver", "fork"])
 def test_processes(server, method):
