@@ -1,11 +1,12 @@
-import pickle
 import json
-
-import pytest
-from fsspec.spec import AbstractFileSystem, AbstractBufferedFile
-import fsspec
+import pickle
 
 import numpy as np
+import pytest
+
+import fsspec
+from fsspec.implementations.ftp import FTPFileSystem
+from fsspec.spec import AbstractFileSystem, AbstractBufferedFile
 
 
 class DummyTestFS(AbstractFileSystem):
@@ -38,6 +39,15 @@ class DummyTestFS(AbstractFileSystem):
         },
         {"name": "misc", "type": "directory"},
         {"name": "misc/foo.txt", "type": "file", "size": 100},
+        {"name": "glob_test", "type": "directory", "size": 0},
+        {"name": "glob_test/hat", "type": "directory", "size": 0},
+        {"name": "glob_test/hat/^foo.txt", "type": "file", "size": 100},
+        {"name": "glob_test/dollar", "type": "directory", "size": 0},
+        {"name": "glob_test/dollar/$foo.txt", "type": "file", "size": 100},
+        {"name": "glob_test/lbrace", "type": "directory", "size": 0},
+        {"name": "glob_test/lbrace/{foo.txt", "type": "file", "size": 100},
+        {"name": "glob_test/rbrace", "type": "directory", "size": 0},
+        {"name": "glob_test/rbrace/}foo.txt", "type": "file", "size": 100},
     )
 
     def __getitem__(self, name):
@@ -60,6 +70,16 @@ class DummyTestFS(AbstractFileSystem):
 
         return list(sorted(files))
 
+    @classmethod
+    def get_test_paths(cls, start_with=""):
+        """Helper to return directory and file paths with no details"""
+        all = [
+            file["name"]
+            for file in cls._fs_contents
+            if file["name"].startswith(start_with)
+        ]
+        return all
+
 
 @pytest.mark.parametrize(
     "test_path, expected",
@@ -101,6 +121,10 @@ class DummyTestFS(AbstractFileSystem):
                 "top_level/second_level/date=2019-10-04/a.parquet",
             ],
         ),
+        ("mock://glob_test/hat/^foo.*", ["glob_test/hat/^foo.txt"]),
+        ("mock://glob_test/dollar/$foo.*", ["glob_test/dollar/$foo.txt"]),
+        ("mock://glob_test/lbrace/{foo.*", ["glob_test/lbrace/{foo.txt"]),
+        ("mock://glob_test/rbrace/}foo.*", ["glob_test/rbrace/}foo.txt"]),
     ],
 )
 def test_glob(test_path, expected):
@@ -115,6 +139,45 @@ def test_glob(test_path, expected):
         assert info == test_fs[name]
 
 
+@pytest.mark.parametrize(
+    ["test_paths", "expected"],
+    [
+        (
+            ("top_level/second_level", "top_level/sec*", "top_level/*"),
+            [
+                "top_level/second_level",
+                "top_level/second_level/date=2019-10-01",
+                "top_level/second_level/date=2019-10-01/a.parquet",
+                "top_level/second_level/date=2019-10-01/b.parquet",
+                "top_level/second_level/date=2019-10-02",
+                "top_level/second_level/date=2019-10-02/a.parquet",
+                "top_level/second_level/date=2019-10-04",
+                "top_level/second_level/date=2019-10-04/a.parquet",
+            ],
+        ),
+        (("misc/foo.txt", "misc/*.txt"), ["misc/foo.txt"]),
+        (
+            ("",),
+            DummyTestFS.get_test_paths() + [DummyTestFS.root_marker],
+        ),
+    ],
+    # ids=["all_second_level", "single_file"],
+)
+def test_expand_path_recursive(test_paths, expected):
+    """Test a number of paths and then their combination which should all yield
+    the same set of expanded paths"""
+    test_fs = DummyTestFS()
+
+    # test single query
+    for test_path in test_paths:
+        paths = test_fs.expand_path(test_path, recursive=True)
+        assert sorted(paths) == sorted(expected)
+
+    # test with all queries
+    paths = test_fs.expand_path(list(test_paths), recursive=True)
+    assert sorted(paths) == sorted(expected)
+
+
 def test_find_details():
     test_fs = DummyTestFS()
     filenames = test_fs.find("/")
@@ -234,3 +297,33 @@ def test_readinto_with_numpy(tmpdir, dt):
         f.readinto(arr2)
 
     assert np.array_equal(arr, arr2)
+
+
+@pytest.mark.parametrize(
+    "dt",
+    [
+        np.int8,
+        np.int16,
+        np.int32,
+        np.int64,
+        np.uint8,
+        np.uint16,
+        np.uint32,
+        np.uint64,
+        np.float32,
+        np.float64,
+    ],
+)
+def test_readinto_with_multibyte(ftp_writable, tmpdir, dt):
+    host, port, user, pw = ftp_writable
+    ftp = FTPFileSystem(host=host, port=port, username=user, password=pw)
+
+    with ftp.open("/out", "wb") as fp:
+        arr = np.arange(10, dtype=dt)
+        fp.write(arr.tobytes())
+
+    with ftp.open("/out", "rb") as fp:
+        arr2 = np.empty_like(arr)
+        fp.readinto(arr2)
+
+    assert np.array_equal(arr, arr2)
